import javax.swing.*;
import java.awt.event.*; 
import java.awt.*; 
import java.io.*;

public class SolfegeGenerator extends JFrame
{
    /*********************************************************************************************/
    
    private JTextField nbNotesField = new JTextField("1000", 5), 
	nbNotesSameKeyField = new JTextField("100", 5),
	maxIntervalField = new JTextField("8", 2), 
	outputFileField = new JTextField("output", 20);

    private JButton go = new JButton("Go !");
    
    private JRadioButton writeInFile = new JRadioButton("Write result in file :");
    private JRadioButton printSource = new JRadioButton("Display Lilypond source code");
    private ButtonGroup outputMode = new ButtonGroup();
    
    private JTextArea resultArea = new JTextArea();    

    private JCheckBox[] keysBoxes;
    private JComboBox[] lowerBounds;
    private JComboBox[] upperBounds;
    
    /*********************************************************************************************/

    public SolfegeGenerator()
    {
	initWindow();
    }

    /*********************************************************************************************/

    private JPanel components(String s, JTextField myField)
    {
	JPanel myPanel = new JPanel();
	myPanel.setLayout(new GridLayout(1, 2)); 
	myPanel.add(new JLabel(s));
	myPanel.add(myField);
	myField.addKeyListener
	    (
	     new KeyListener()
	     {
		 public void keyTyped(KeyEvent e){} 	
		 public void keyPressed(KeyEvent e){} 
		 public void keyReleased(KeyEvent e)
		 { 
		     maybeEnableGo();
		 }
	     }
	     );
	return myPanel;
    }

    /*********************************************************************************************/

    private void selectNote(JComboBox b, String s)
    {	
	int i = - 1;
	Object o;
	do
	    {
		i++;
		o = b.getItemAt(i);
	    }
	while(!((String)o).equals(s));
	b.setSelectedIndex(i);
    }
    
    /*********************************************************************************************/

    private JPanel addKeys()
    {	
	keysBoxes = new JCheckBox[7];
	lowerBounds = new JComboBox[7];
	upperBounds = new JComboBox[7];
	for (int i = 0 ; i < 7 ; i++)
	    {
		lowerBounds[i] = new JComboBox();
		upperBounds[i] = new JComboBox();
		
		for (int octava = 5 ; octava >=0 ; octava--)
		    {		
			for (char note = 'b' ; note >= 'a' ; note--)
			    {
				lowerBounds[i].addItem("" + note + octava);
				upperBounds[i].addItem("" + note + octava);
			    }
			for (char note = 'g' ; note >= 'c' ; note--)
			    {
				lowerBounds[i].addItem("" + note + octava);
				upperBounds[i].addItem("" + note + octava);
			    }
		    }
	    }
	keysBoxes[0] = new JCheckBox("g2"); 
	selectNote(lowerBounds[0], "g2"); 	selectNote(upperBounds[0], "g5");
	keysBoxes[1] = new JCheckBox("f4"); 
	selectNote(lowerBounds[1], "g0"); 	selectNote(upperBounds[1], "g3");
	keysBoxes[2] = new JCheckBox("f3"); 
	selectNote(lowerBounds[2], "c2"); 	selectNote(upperBounds[2], "g4");
	keysBoxes[3] = new JCheckBox("c4"); 
	selectNote(lowerBounds[3], "c2"); 	selectNote(upperBounds[3], "c4");
	keysBoxes[4] = new JCheckBox("c3"); 
	selectNote(lowerBounds[4], "c2"); 	selectNote(upperBounds[4], "c4");
	keysBoxes[5] = new JCheckBox("c2"); 
	selectNote(lowerBounds[5], "c2"); 	selectNote(upperBounds[5], "c4");
	keysBoxes[6] = new JCheckBox("c1"); 
	selectNote(lowerBounds[6], "c2"); 	selectNote(upperBounds[6], "c4");
	JPanel keysBoxesPanel = new JPanel();
	keysBoxesPanel.setLayout(new GridLayout(8, 3));
	keysBoxesPanel.add(new JLabel("keys "));
	keysBoxesPanel.add(new JLabel("lowest note "));
	keysBoxesPanel.add(new JLabel("highest note "));
	for (int i = 0 ; i < 7 ; i++)
	    {
		keysBoxesPanel.add(keysBoxes[i]);
		keysBoxesPanel.add(lowerBounds[i]);
		keysBoxesPanel.add(upperBounds[i]);
		selectKey(i, false);
		final int j = i;
		keysBoxes[i].addActionListener
		    (
		     new ActionListener()
		     {
			 public void actionPerformed(ActionEvent e)
			 {
			     selectKey(j, keysBoxes[j].isSelected());
			 }
		     }
		     );
		    }
	selectKey(0, true);
	return keysBoxesPanel;
    }    
    
    /*********************************************************************************************/

    private void selectKey(int keyIndex, boolean b)
    {
	keysBoxes[keyIndex].setSelected(b);
	lowerBounds[keyIndex].setEnabled(b);
	upperBounds[keyIndex].setEnabled(b);
	int nbSelected = 0;
	for (int i = 0 ; i < 7 ; i++)
	    if (keysBoxes[i].isSelected())
		nbSelected++;
	nbNotesSameKeyField.setEnabled(nbSelected > 1);
	maybeEnableGo();
    }
    
    /*********************************************************************************************/

    private void maybeEnableGo()
    {
	int nbSelected = 0;
	for (int i = 0 ; i < 7 ; i++)
	    if (keysBoxes[i].isSelected())
		nbSelected++;
	try
	    {
		go.setEnabled
		    (nbSelected != 0 
		     && Integer.parseInt(nbNotesField.getText()) >= 1 
		     && (Integer.parseInt(nbNotesSameKeyField.getText()) >= 1 || nbSelected == 1)
		     && Integer.parseInt(maxIntervalField.getText()) >= 1
		     && (outputFileField.getText().length() != 0 || printSource.isSelected())
		     );
	    }
	catch(NumberFormatException e)
	    {
		go.setEnabled(false);
	    }
    }

    /*********************************************************************************************/

    private void writeResult(String s)
    {
	resultArea.append(s);
    }

    /*********************************************************************************************/

    private JPanel addButtons()
    {
	JPanel buttonsPanel = new JPanel();
	buttonsPanel.setLayout(new FlowLayout());
	JButton quit = new JButton("Quit");
	quit.addActionListener
	    (
	     new ActionListener()
	     {
		 public void actionPerformed(ActionEvent e)
		 {
		     dispose();
		 }
	     }	     
	     );	
	buttonsPanel.add(quit);
	go.addActionListener
	    (
	     new ActionListener()
	     {
		 public void actionPerformed(ActionEvent e)
		 {
		     String s = "hemiole --nb " + nbNotesField.getText() +
			 " --i " + maxIntervalField.getText() + 
			 " --max " + nbNotesSameKeyField.getText() +
			 getKeys();
		     if (writeInFile.isSelected())
			 s += " --o " + outputFileField.getText();
		     else
			 s += " --o stdout";
		     final String j = s;
		     writeResult("Processing command "+ j + "\nPlease wait\n");
		     Runnable rErrorStream = 
			 (
			  new Runnable()
			  {
			      public void run()
			      {
				  try
				      {
					  Process findspace;
					  findspace = Runtime.getRuntime().exec(j);
					  BufferedReader resultset = new BufferedReader
					      (new InputStreamReader
					       (findspace.getErrorStream()
						)
					       );					  
					  int c;
					  while ((c = resultset.read()) != -1) {
					      writeResult("" + (char)c);
					  }
				      }
				  catch (Exception f)
				      {
					  writeResult("Oups... " + f + "\n");
				      }
			      }
			  }
			  );
		     (new Thread(rErrorStream)).start();
		     Runnable rInputStream = 
			 (
			  new Runnable()
			  {
			      public void run()
			      {
				  try
				      {
					  Process findspace;
					  findspace = Runtime.getRuntime().exec(j);
					  BufferedReader resultset = new BufferedReader
					      (new InputStreamReader
					       (findspace.getInputStream()
						)
					       );					  
					  int c;
					  while ((c = resultset.read()) != -1) {
					      writeResult("" + (char)c);
					  }
				      }
				  catch (Exception f)
				      {
					  writeResult("Oups... " + f + "\n");
				      }
			      }
			  }
			  );
		     (new Thread(rInputStream)).start();
		 }
	     }	     
	     );
	buttonsPanel.add(go);
	return buttonsPanel;
    }    

    /*********************************************************************************************/

    private JPanel outputMode()
    {
	JPanel myPanel = new JPanel();
	outputMode.add(writeInFile);
	outputMode.add(printSource);
	myPanel.setLayout(new GridLayout(2, 2));
	writeInFile.setSelected(true);
	writeInFile.addActionListener
	    (
	     new ActionListener()
	     {
		 public void actionPerformed(ActionEvent e)
		 {
		     outputFileField.setEnabled(true);
		     maybeEnableGo();
		 }
	     }
	     );
	printSource.addActionListener
	    (
	     new ActionListener()
	     {
		 public void actionPerformed(ActionEvent e)
		 {
		     outputFileField.setEnabled(false);
		     maybeEnableGo();
		 }
	     }
	     );
	outputFileField.addKeyListener
	    (
	     new KeyListener()
	     {
		 public void keyTyped(KeyEvent e){} 	
		 public void keyPressed(KeyEvent e){} 
		 public void keyReleased(KeyEvent e)
		 { 
		     maybeEnableGo();
		 }
	     }
	     );
	myPanel.add(writeInFile);
	myPanel.add(outputFileField);
	myPanel.add(printSource);
	return myPanel;
    }
    
    /*********************************************************************************************/

    private String getKeys()
    {
	String s = "";
	for(int i = 0 ; i < 7 ; i++)
	    {
		if (keysBoxes[i].isSelected())
		    {
			s += " --key " + keysBoxes[i].getText()
			    + " " + lowerBounds[i].getSelectedItem().toString()
			    + " " + upperBounds[i].getSelectedItem().toString();
		    }
	    }
	return s;
    }

    /*********************************************************************************************/

    private JPanel addOutputPanel()
    {
	JPanel outputPanel = new JPanel();	
	outputPanel.setLayout(new FlowLayout());
	resultArea.setColumns(80);
	resultArea.setRows(20);
	resultArea.setLineWrap(true);
	resultArea.setEditable(false);
	JScrollPane myScroll = new JScrollPane(resultArea);
	outputPanel.add(myScroll);
	return outputPanel;
    }
    
    /*********************************************************************************************/

    private void initWindow()
    {
	setTitle("J-Hemiole");
	setSize(900, 700);
	setResizable(false);
	setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	getContentPane().setLayout(new BoxLayout(getContentPane(), BoxLayout.Y_AXIS)); 
	getContentPane().add(components("Number of notes ", nbNotesField));
	getContentPane().add(components("Number of notes with the same key ", nbNotesSameKeyField));
	getContentPane().add(components("Greater interval ", maxIntervalField));
	getContentPane().add(addKeys());
	getContentPane().add(outputMode());
	getContentPane().add(addButtons());
	getContentPane().add(addOutputPanel());
	setVisible(true);
    } 
    
    /*********************************************************************************************/

    public static void main(String[] args)
    {
	SolfegeGenerator s = new SolfegeGenerator();
    }
}
